Type: Package
Package: TriMatch
Title: Propensity Score Matching of Non-Binary Treatments
Version: 1.0.2
Date: 2026-04-28
Authors@R: 
    person("Jason", "Bryer", , "jason@bryer.org", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-2454-0402"))
Description: Propensity score matching for non-binary treatments.
License: GPL (>= 2)
URL: https://jbryer.github.io/TriMatch/,
        https://github.com/jbryer/TriMatch/
BugReports: https://github.com/jbryer/TriMatch/issues/
Depends: ggplot2, R (>= 3.0), reshape2, scales
Imports: car, compiler, grid, gridExtra, plyr, PSAgraphics, psych,
        randomForest, stats, stringr
Suggests: bookdown, knitr, MASS, rmarkdown, xtable
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-28 16:19:14 UTC; jasonbryer
Author: Jason Bryer [aut, cre] (ORCID: <https://orcid.org/0000-0002-2454-0402>)
Maintainer: Jason Bryer <jason@bryer.org>
Repository: CRAN
Date/Publication: 2026-04-28 16:50:21 UTC
Built: R 4.5.2; ; 2026-05-01 19:39:51 UTC; unix
