# Generated by roxygen2: do not edit by hand

S3method(merge,triangle.matches)
S3method(merge,triangle.psa)
S3method(plot,balance.plots)
S3method(plot,triangle.matches)
S3method(plot,triangle.psa)
S3method(print,balance.plots)
S3method(print,triangle.plot)
S3method(print,trimatch.summary)
S3method(summary,balance.plots)
S3method(summary,triangle.matches)
S3method(summary,triangle.psa)
S3method(summary,unmatched)
export(OneToN)
export(balance.plot)
export(boxdiff.plot)
export(covariateBalance)
export(data.frame.to.list)
export(distances.plot)
export(loess3.plot)
export(maximumTreat)
export(multibalance.plot)
export(parallel.plot)
export(perpPt)
export(segment1)
export(segment2)
export(trimatch)
export(trips)
export(unmatched)
import(PSAgraphics)
import(compiler)
import(ggplot2)
import(gridExtra)
import(plyr)
import(reshape2)
import(stats)
importFrom(car,Anova)
importFrom(graphics,plot)
importFrom(grid,editGrob)
importFrom(grid,gTree)
importFrom(grid,getGrob)
importFrom(grid,grid.draw)
importFrom(grid,grid.layout)
importFrom(grid,grid.newpage)
importFrom(grid,grob)
importFrom(grid,grobHeight)
importFrom(grid,grobWidth)
importFrom(grid,pushViewport)
importFrom(grid,upViewport)
importFrom(grid,viewport)
importFrom(grid,vpPath)
importFrom(grid,vpTree)
importFrom(psych,describe)
importFrom(psych,describeBy)
importFrom(randomForest,randomForest)
importFrom(scales,percent)
importFrom(stringr,str_count)
importFrom(stringr,str_detect)
