% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/star.R
\name{star}
\alias{star}
\title{Returns significance level.}
\usage{
star(x)
}
\arguments{
\item{x}{p-value.}
}
\description{
Returns the significance level as stars, or NA if a non-numeric value is
passed in.
}
